import json
import locale
from pathlib import Path

from hohu.config import get_lang


class I18n:
    def __init__(self):
        self.lang = get_lang()
        self.locales = {}
        self._load_all_locales()

    def _load_all_locales(self):
        """动态加载 locales 文件夹下的所有 json 文件"""
        # 获取当前文件所在目录下的 locales 文件夹
        locales_dir = Path(__file__).parent / "locales"

        if not locales_dir.exists():
            # 兼容性处理：如果找不到，尝试在当前工作目录找（开发调试用）
            locales_dir = Path.cwd() / "hohu" / "locales"

        for json_file in locales_dir.glob("*.json"):
            try:
                with open(json_file, encoding="utf-8") as f:
                    # 使用文件名作为 key (例如 'en', 'zh')
                    self.locales[json_file.stem] = json.load(f)
            except Exception:
                continue

        # 兜底：如果连 en 都没加载到，给一个空字典防止 KeyError
        if "en" not in self.locales:
            self.locales["en"] = {}

    def t(self, key: str) -> str:
        """获取翻译，支持 auto 跟随系统"""
        target_lang = self.lang

        if target_lang == "auto":
            sys_lang = locale.getdefaultlocale()[0]
            target_lang = "zh" if sys_lang and "zh" in sys_lang else "en"

        # 优先找目标语言，找不到找英文，再找不到返回 key 本身
        lang_data = self.locales.get(target_lang, self.locales.get("en", {}))
        return lang_data.get(key, key)

    def refresh(self):
        """当语言配置改变时调用"""
        self.lang = get_lang()


i18n = I18n()



import shutil
import subprocess

from rich.console import Console

from hohu.i18n import i18n
from hohu.utils.project import ProjectManager

console = Console()


def init():
    """Initialize environment for the current project (uv/pnpm)"""
    root = ProjectManager.find_root()
    if not root:
        console.print(f"[red]{i18n.t('not_in_project')}[/red]")
        return

    info = ProjectManager.get_info(root)
    console.print(f"🛠️  [bold]Initializing: {info['name']}[/bold]\n")

    for item in info["components"]:
        folder = {
            "Backend": "hohu-admin",
            "Frontend": "hohu-admin-web",
            "App": "hohu-admin-app",
        }[item]
        path = root / folder

        if not path.exists():
            continue

        if item == "Backend":
            install_cmd = (
                ["uv", "sync"]
                if shutil.which("uv")
                else ["pip", "install", "-r", "requirements.txt"]
            )
            console.print(
                f"📦 [dim]Executing {' '.join(install_cmd)} in {folder}...[/dim]"
            )
            subprocess.run(install_cmd, cwd=path)

            init_script = path / "scripts" / "init.py"
            if init_script.exists():
                console.print(
                    f"🚀 [dim]Running initialization script: {init_script.name}...[/dim]"
                )
                subprocess.run(["python", "scripts/init.py"], cwd=path)
            else:
                console.print(
                    f"[yellow]⚠️  Init script not found at {init_script}[/yellow]"
                )
        else:
            cmd = ["pnpm", "install"] if shutil.which("pnpm") else ["npm", "install"]
            console.print(f"📦 [dim]Executing {' '.join(cmd)} in {folder}...[/dim]")
            subprocess.run(cmd, cwd=path)

    console.print(
        f"\n✅ {i18n.t('init_success')} {i18n.t('dev_start')}: [bold cyan]hohu admin dev[/bold cyan]"
    )



import signal
import subprocess
import sys
import threading

import typer
from rich.console import Console

from hohu.i18n import i18n
from hohu.utils.project import ProjectManager

console = Console()


def log_worker(pipe, prefix, color):
    """
    后台线程：负责读取进程的 stdout/stderr 并添加彩色前缀
    """
    try:
        with pipe:
            for line in iter(pipe.readline, b""):
                # 解码并去除末尾换行
                message = line.decode("utf-8", errors="replace").strip()
                if message:
                    # 使用 Rich 打印带颜色前缀的日志
                    console.print(f"[{color}][{prefix}][/{color}] {message}")
    except Exception as e:
        console.print(f"[red]日志流异常 ({prefix}): {e}[/red]")


def dev(
    target: str = typer.Option(
        "h5", "--app-target", "-t", help="APP端目标: h5, mp, app"
    ),
    only: list[str] | None = typer.Option(
        None, "--only", "-o", help="仅启动指定组件(支持简写: be, fe, app)"
    ),
    skip: list[str] | None = typer.Option(None, "--skip", "-s", help="跳过指定组件"),
):
    """
    启动开发环境。支持通过 --only 或 --skip 过滤组件。
    """
    root = ProjectManager.find_root()
    if not root:
        console.print(f"[red]{i18n.t('not_in_project')}[/red]")
        return

    info = ProjectManager.get_info(root)
    # 获取当前项目拥有的组件
    available_components = info["components"]
    # 定义简写映射表 (全小写)
    alias_map = {
        "be": "backend",
        "backend": "backend",
        "admin": "backend",
        "fe": "frontend",
        "frontend": "frontend",
        "web": "frontend",
        "app": "app",
    }

    def normalize(names: list[str]) -> set:
        """将用户的各种输入映射回标准的组件名称"""
        normalized = set()
        for n in names:
            name_low = n.lower()
            if name_low in alias_map:
                # 转换回标准的首字母大写格式以匹配 available_components
                standard_name = alias_map[name_low].capitalize()
                normalized.add(standard_name)
        return normalized

    # 处理过滤逻辑
    only_set = normalize(only) if only else set()
    skip_set = normalize(skip) if skip else set()

    to_run = []
    for item in available_components:
        # 如果指定了 only，则只运行命中项
        if only_set and item not in only_set:
            continue
        # 如果指定了 skip，则排除命中项
        if skip_set and item in skip_set:
            continue
        to_run.append(item)

    if not to_run:
        console.print(
            "[yellow]没有需要启动的组件。请检查 --only 或 --skip 参数。[/yellow]"
        )
        console.print(f"[dim]已安装组件: {available_components}[/dim]")
        return

    processes = []
    console.print(f"🚀 [bold magenta]Starting: {', '.join(to_run)}[/bold magenta]\n")
    console.print("💡 [dim]Press Ctrl+C to stop all services[/dim]\n")

    # 定义组件配置
    config_map = {
        "Backend": {
            "folder": "hohu-admin",
            "cmd": ["uv", "run", "fastapi", "dev", "app/main.py"],
            "color": "green",
        },
        "Frontend": {
            "folder": "hohu-admin-web",
            "cmd": ["pnpm", "dev"],
            "color": "cyan",
        },
        "App": {
            "folder": "hohu-admin-app",
            "cmd": ["pnpm", "dev" if target == "h5" else f"dev:{target}"],
            "color": "yellow",
        },
    }

    # 启动进程
    for item in to_run:
        conf = config_map.get(item)
        if not conf:
            continue

        cwd = root / conf["folder"]
        if not cwd.exists():
            console.print(f"[red]目录不存在: {cwd}[/red]")
            continue

        try:
            # 开启子进程，并重定向 stdout 和 stderr
            process = subprocess.Popen(
                conf["cmd"],
                cwd=cwd,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,  # 合并错误流
                bufsize=1,  # 行缓冲
                env=None,  # 可以按需传入 os.environ
            )
            processes.append(process)

            # 为每个进程启动一个守护线程来读取输出
            t = threading.Thread(
                target=log_worker,
                args=(process.stdout, item, conf["color"]),
                daemon=True,
            )
            t.start()

        except Exception as e:
            console.print(f"[bold red]无法启动 {item}: {e}[/bold red]")

    # 处理退出逻辑
    def signal_handler(_sig, _frame):
        console.print("\n[bold yellow]正在停止所有服务...[/bold yellow]")
        for p in processes:
            p.terminate()
        sys.exit(0)

    # 注册 Ctrl+C 信号
    signal.signal(signal.SIGINT, signal_handler)

    # 保持主线程运行
    try:
        for p in processes:
            p.wait()
    except KeyboardInterrupt:
        signal_handler(None, None)



import json
from pathlib import Path


class ProjectManager:
    """处理当前项目目录下的 .hohu 追踪逻辑"""

    @staticmethod
    def find_root() -> Path | None:
        """向上查找包含 .hohu 文件夹的根目录"""
        for parent in [Path.cwd()] + list(Path.cwd().parents):
            if (parent / ".hohu").is_dir() and (parent / ".hohu/project.json").exists():
                return parent
        return None

    @staticmethod
    def mark_project(root: Path, name: str, components: list):
        """在项目根目录创建标识"""
        dot_hohu = root / ".hohu"
        dot_hohu.mkdir(exist_ok=True)
        data = {"name": name, "components": components}
        (dot_hohu / "project.json").write_text(json.dumps(data, indent=4))

    @staticmethod
    def get_info(root: Path) -> dict:
        return json.loads((root / ".hohu/project.json").read_text())



import questionary
import typer
from rich.console import Console
from rich.table import Table

from hohu import __version__
from hohu.config import CONFIG_FILE, load_config, save_config
from hohu.i18n import i18n

console = Console()
system_app = typer.Typer(help="System settings & Information")


@system_app.command("info")
def show_info():
    """View system configuration information"""
    config = load_config()

    table = Table(
        title="HoHu CLI System Configuration",
        show_header=False,
        title_style="bold magenta",
    )

    table.add_row("Version", f"[green]{__version__}[/green]")
    table.add_row("Language", f"[cyan]{config.get('language', 'auto')}[/cyan]")
    table.add_row("Config Path", f"[dim]{CONFIG_FILE}[/dim]")
    table.add_row("I18n Status", "[green]Loaded[/green]")

    console.print(table)


@system_app.command("lang")
def set_language():
    """Set the CLI interaction language"""
    choices = [
        questionary.Choice("简体中文", value="zh"),
        questionary.Choice("English", value="en"),
        questionary.Choice("Follow System (跟随系统)", value="auto"),
    ]

    selected_lang = questionary.select(i18n.t("select_lang"), choices=choices).ask()

    if selected_lang:
        config = load_config()
        config["language"] = selected_lang
        save_config(config)

        # 假设 i18n 实例有 refresh 方法
        i18n.refresh()
        console.print(f"[green]✔[/green] {i18n.t('lang_updated')}")



import typer

from hohu.commands.admin.create import create
from hohu.commands.admin.dev import dev
from hohu.commands.admin.init import init

admin_app = typer.Typer(help="Admin commands for projects")

admin_app.command(name="create")(create)
admin_app.command(name="init")(init)
admin_app.command(name="dev")(dev)



import subprocess
from pathlib import Path

import questionary
import typer
from rich.console import Console

from hohu.i18n import i18n
from hohu.utils.project import ProjectManager

console = Console()

REPOS = {
    "Backend": "https://github.com/aihohu/hohu-admin.git",
    "Frontend": "https://github.com/aihohu/hohu-admin-web.git",
    "App": "https://github.com/aihohu/hohu-admin-app.git",
}


def create(project_name: str = typer.Argument("hohu-admin")):
    """Create a new project directory and clone templates"""
    root = Path.cwd() / project_name
    if root.exists():
        console.print(f"[red]Error: {project_name} already exists.[/red]")
        return

    choices = questionary.checkbox(
        i18n.t("select_components"),
        choices=[
            questionary.Choice("Backend", checked=True),
            questionary.Choice("Frontend", checked=True),
            questionary.Choice("App", checked=True),
        ],
    ).ask()

    if not choices:
        return

    try:
        root.mkdir(parents=True)
        ProjectManager.mark_project(root, project_name, choices)

        for item in choices:
            folder = {
                "Backend": "hohu-admin",
                "Frontend": "hohu-admin-web",
                "App": "hohu-admin-app",
            }[item]
            console.print(f"🚚 [blue]{i18n.t('cloning')} {item}...[/blue]")
            subprocess.run(
                ["git", "clone", REPOS[item], str(root / folder)], check=True
            )

        console.print(
            f"\n✨ {i18n.t('success_msg')} [bold cyan]cd {project_name} && hohu admin init[/bold cyan]"
        )
    except Exception as e:
        console.print(f"[red]Failed: {e}[/red]")



import typer
from rich.console import Console

from hohu import __version__
from hohu.commands.admin import admin_app
from hohu.commands.system import set_language, show_info, system_app

app = typer.Typer(name="hohu", help="HoHu CLI Tool", no_args_is_help=True)
console = Console()


def version_callback(value: bool):
    if value:
        from rich.console import Console

        console = Console()
        console.print(
            f"🚀 [bold cyan]HoHu CLI[/bold cyan] Version: [green]{__version__}[/green]"
        )
        raise typer.Exit()


@app.callback()
def main(
    version: bool = typer.Option(
        None,
        "--version",
        "-v",
        callback=version_callback,
        is_eager=True,
        help="Show version",
    ),
):
    """HoHu 全能开发管理工具"""
    pass


app.add_typer(admin_app, name="admin")
app.add_typer(system_app, name="system")

app.command(name="lang")(set_language)
app.command(name="info")(show_info)

if __name__ == "__main__":
    app()



import json
from pathlib import Path

# 全局配置目录：~/.hohu/
CONFIG_DIR = Path.home() / ".hohu"
CONFIG_FILE = CONFIG_DIR / "config.json"

DEFAULT_CONFIG = {"language": "auto"}


def load_config():
    if not CONFIG_FILE.exists():
        return DEFAULT_CONFIG
    try:
        with open(CONFIG_FILE, encoding="utf-8") as f:
            return json.load(f)
    except Exception:
        return DEFAULT_CONFIG


def save_config(config: dict):
    CONFIG_DIR.mkdir(parents=True, exist_ok=True)
    with open(CONFIG_FILE, "w", encoding="utf-8") as f:
        json.dump(config, f, indent=4, ensure_ascii=False)


def get_lang():
    return load_config().get("language", "auto")
